@@ -2,6 +2,7 @@ library(nanonext)
 nanotest <- function(x) invisible(x || stop("is not TRUE when expected to be TRUE"))
 nanotestw <- function(x) invisible(suppressWarnings(x) || stop("is not TRUE when expected to be TRUE"))
 nanotestz <- function(x) invisible(x == 0L || stop("does not equal 0L as expected"))
+nanotesti <- function(a, b) invisible(identical(a, b) || stop("the arguments are not identical as expected"))
 nanotestaio <- function(x) invisible(is_aio(x) || stop("is not of class 'recvAio' or 'sendAio' as expected"))
 nanotestnano <- function(x) invisible(is_nano(x) || stop("does not inherit from class 'nano' as expected"))
 nanotestn <- function(x) invisible(is.null(x) || stop("is not NULL when expected to be NULL"))
@@ -73,7 +74,7 @@ nanotest(is_error_value(raio$data))
 nanotestz(n$send(data.frame(), block = FALSE))
 nanotest(is.data.frame(n1$recv(block = 500)))
 nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
-nanotest(identical(n$recv("character", block = 500), c("test", "", "spec")))
+nanotesti(n$recv("character", block = 500), c("test", "", "spec"))
 nanotestz(n$send(1:5, mode = "r"))
 nanotest(length(n1$recv("int", block = 500)) == 5L)
 nanotestaio(saio <- n1$send_aio(Reduce(strcat, replicate(5, random(1e3L))), mode = 3L, timeout = 900))
@@ -91,16 +92,16 @@ nanotestaio(saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500))
 saio$newfield <- "doesnotwork"
 saio[["newfield"]] <- "doesnotwork"
 nanotestn(saio$newfield)
-nanotest(is.logical(.unresolved(saio)))
 nanotest(is.logical(unresolved(saio)))
+nanotest(is.logical(.unresolved(saio)))
 nanotestaio(msg <- n1$recv_aio(mode = "numer", timeout = 500))
-nanotest(identical(call_aio(msg), msg))
+nanotesti(call_aio(msg), msg)
 nanotestaio(msg <- n1$recv_aio(mode = "complex", timeout = 500))
 nanotestn(stop_aio(msg))
 nanotestn(stop_aio(n))
-nanotest(identical(call_aio(msg), msg))
+nanotesti(call_aio(msg), msg)
 nanotest(is_error_value(msg$data))
-nanotest(identical(call_aio(n), n))
+nanotesti(call_aio(n), n)
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = 1L, timeout = 500))
 nanotest(is_nul_byte(call_aio_(rraio)$data))
@@ -160,13 +161,22 @@ nanotestw(n1$close() == 7L)
 nanotest(n$socket[["state"]] == "closed")
 nanotest(n1$socket[state] == "closed")
 
+nanotest(inherits(cv <- cv(), "conditionVariable"))
+nanotestp(cv)
+nanotestxp(cv2 <- cv())
+nanotest(!until(cv, 10L))
+nanotest(!until(cv, 10))
+nanotest(!until_(cv, 10L))
+nanotest(!until_(cv, 10))
+nanotest(!until_(cv, "test"))
+nanotestz(cv_reset(cv))
+nanotestz(cv_value(cv))
+
 nanotestnano(req <- nano("req", listen = "inproc://testing"))
 nanotestnano(rep <- socket("rep", dial = "inproc://testing", listen = "inproc://testing2"))
 nanotestp(rep)
-if (nng_version()[1] == "1.6.0pre") {
-  nanotest(stat(rep, "dialers") == 1)
-  nanotest(stat(rep, "protocol") == "rep")
-}
+nanotest(stat(rep, "dialers") == 1)
+nanotest(stat(rep, "protocol") == "rep")
 nanotestn(stat(rep, "nonexistentstat"))
 nanotestnano(req$opt("req:resend-time", 1000))
 nanotest(req$opt("req:resend-time") == 1000L)
@@ -202,15 +212,21 @@ nanotest(!unresolved(msg))
 nanotest(is.integer(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500)))
 nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 nanotest(is.logical(call_aio(msg)$data))
+nanotesti(collect_aio(msg), collect_aio_(msg))
 nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "next"))
 nanotestn(stop_aio(err))
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
+nanotest(is_error_value(call_aio(list(err))[[1L]][["result"]]))
 nanotest(is_error_value(call_aio_(err)$result))
+nanotest(is_error_value(call_aio_(list(item = err))[["item"]][["result"]]))
+nanotest(is_error_value(collect_aio(err)))
+nanotest(is_error_value(collect_aio(list(item = err))[["item"]]))
+nanotest(is_error_value(collect_aio_(list(err))[[1L]]))
 nanotest(is.integer(req$send(serialize(NULL, NULL, ascii = TRUE), mode = 2L, block = 500)))
 nanotestn(call_aio(recv_aio(ctx, mode = 1L, timeout = 500))[["value"]])
 nanotestaio(saio <- send_aio(ctx, as.raw(1L), mode = 2L, timeout = 500))
-nanotest(identical(req$recv(mode = 8L, block = 500), as.raw(1L)))
+nanotesti(req$recv(mode = 8L, block = 500), as.raw(1L))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
 nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
@@ -226,28 +242,21 @@ nanotest(is.integer(req$send(custom, mode = 3L, block = 500)))
 nanotest(is.integer(recv(rep, block = 500)[[1L]]))
 nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = TRUE)))
 nanotest(is.integer(send(rep, custom, mode = 3L, block = 500)))
-nanotest(identical(next_config(nxt), nxt))
-nanotest(identical(next_config(as.pairlist(c("a", "b", "c"))), nxt))
+nanotesti(next_config(nxt), nxt)
+nanotesti(next_config(as.pairlist(c("a", "b", "c"))), nxt)
 nanotest(is.list(next_config(NULL, mark = TRUE)))
 nanotestn(unlist(next_config()))
 
-nanotest(inherits(cv <- cv(), "conditionVariable"))
-nanotestp(cv)
-nanotestxp(cv2 <- cv())
-nanotest(!until(cv, 10L))
-nanotest(!until(cv, 10))
-nanotest(!until_(cv, 10L))
-nanotest(!until_(cv, 10))
-nanotestz(cv_reset(cv))
-nanotestz(cv_value(cv))
 nanotestaio(cs <- request_signal(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
+nanotestaio(set_promise_context(cs, environment()))
+nanotestnn(cs$data)
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
 nanotestaio(cr <- recv_aio_signal(ctxn, cv = cv, timeout = 500))
 nanotest(call_aio(cr)$data == "test")
 nanotest(is.integer(send(ctxn, TRUE, mode = 0L, block = FALSE)))
-nanotest(is.integer(recv(ctxn, mode = 8L, block = FALSE)))
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
 nanotestaio(cs <- request(.context(req$socket), data = TRUE))
+nanotestnn(cs$data)
 nanotest(recv(ctxn, block = 500))
 nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
 nanotestz(reap(ctxn))
@@ -258,8 +267,6 @@ nanotestz(pipe_notify(req$socket, cv = cv, add = TRUE))
 nanotestz(pipe_notify(req$socket, cv = cv, cv2 = cv2, remove = TRUE, flag = tools::SIGCONT))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
 nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
-nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
-nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
 nanotesterr(wait(err), "valid")
 nanotesterr(wait_(err), "valid")
 nanotesterr(until(err, 10), "valid")
@@ -358,6 +365,7 @@ nanotest(is_error_value(ncurl("http")$data))
 haio <- ncurl_aio("http://trin.cam.ac.uk/")
 nanotest(length(call_aio(haio)$headers))
 haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
+nanotestnn(haio$status)
 nanotest(is.integer(call_aio(haio)$status))
 put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("Date", "server"), timeout = 3000L)
 nanotestp(put1)
@@ -418,7 +426,6 @@ for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
 nanotest(strcat("hello ", "world") == "hello world")
 
-nanotestxp(cv <- cv())
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
 if (length(s)) {
   nanotestnn(recv(s, block = 500L))
@@ -439,7 +446,6 @@ if (length(s)) {
   nanotestaio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv))
   nanotest(length(call_aio_(sr)[["data"]]))
   nanotest(cv_value(cv) > 0L)
-  nanotest(wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
   nanotestnn(opt(s, "tcp-nodelay") <- FALSE)
   nanotesterr(recv(s, mode = "none", block = FALSE), "mode")
@@ -465,6 +471,7 @@ nanotestnn(recv(s3, block = 500))
 nanotesterr(unlock(cv), "valid Socket")
 nanotesterr(lock(cv), "valid Socket")
 nanotesterr(lock(s, cv = s), "valid Condition Variable")
+nanotestz(pipe_notify(s, cv = NULL, add = TRUE, remove = TRUE))
 nanotestz(close(s))
 nanotestz(close(s1))
 nanotestz(close(s2))
@@ -514,13 +521,20 @@ nanotesterr(close(fakelist), "valid Listener")
 unres <- `class<-`(NA, "unresolvedValue")
 nanotest(!unresolved(unres))
 nanotestp(unres)
-nanotest(identical(call_aio("a"), "a"))
-nanotest(identical(call_aio_("a"), "a"))
+nanotest(is.logical(unres <- unresolved(list("a", "b"))) && length(unres) == 1L)
+nanotest(is.integer(unres <- .unresolved(list("a", "b"))) && length(unres) == 1L)
+nanotesti(call_aio("a"), "a")
+nanotesti(call_aio_("a"), "a")
+nanotesterr(collect_aio_("a"), "object is not an Aio or list of Aios")
+nanotesterr(collect_aio_(list("a")), "object is not an Aio or list of Aios")
+nanotesterr(collect_aio(list(fakesock)), "object is not an Aio or list of Aios")
 nanotestn(stop_aio("a"))
+nanotestn(stop_aio(list("a")))
 nanotest(is.environment(set_promise_context(new.env(), new.env())))
 
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
+nanotest(is.character(base64enc(c("vector", "test"))))
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
 nanotest(is.raw(base64dec(base64enc(as.raw(c(1L, 2L)), convert = FALSE), convert = FALSE)))
 nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
@@ -540,7 +554,7 @@ nanotesterr(tls_config(client = c(pem, pem)), "Cryptographic error")
 nanotesterr(tls_config(server = c(pem, pem)), "Cryptographic error")
 nanotest(is.list(cert <- write_cert(cn = "127.0.0.1")))
 nanotest(length(cert) == 2L && is.character(cert[[1L]]))
-nanotest(identical(names(cert), c("server", "client")))
+nanotesti(names(cert), c("server", "client"))
 nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
@@ -555,7 +569,9 @@ nanotestw(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestw(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestz(close(s1))
 nanotestz(close(s))
-rm(list = ls())
-gc()
-Sys.sleep(1L)
-.Call(nanonext:::rnng_fini)
+if (Sys.info()[["sysname"]] == "Linux") {
+  rm(list = ls())
+  gc()
+  Sys.sleep(1L)
+  .Call(nanonext:::rnng_fini)
+}
